/*
 * cmdFile is used below since CCS/project wizard does not allow the
 * use of linkerCommandFile like we'd like.  The workaround is to provide
 * the .cmd file in the fileList[] array and set linkerCommandFile to "".
 */

/*
 *  Remove C6472 and TI816X from generic C6000, since these will be covered by
 *  other filters.
 */
var filterGenericC6000 = [
    {deviceFamily: "C6000",
     deviceId: "~.*(TMS320C670|TMS320C671|TMS320C672|DSK6713|GenericC62xxDevice|GenericC64xxDevice|GenericC67xxDevice|GenericC67xPlusDevice|C641|C6472|TI816x).*",
     toolChain: "TI"},
];

/*
 *  Remove TI816x from generic ARM, since it will be covered by
 *  other filters.
 */
var filterGenericARM = [
    {deviceFamily:"ARM", deviceId: "~.*(Cortex A|TM4C|TM4E|TM4L|LM3|F28M3|LM4|RM4|TMS570LS|ARM7|ARM11|Generic|EVMDMRX45X|CC2538|TMS470M|DM350|DM357|DM368|TI816x).*", toolChain: "TI"},
];

var filterA8 = [
    {deviceFamily:"ARM", deviceVariant:"CortexA8", toolChain:"TI"},
];

var filterA8GNU = [
    {deviceFamily:"ARM", deviceVariant:"CortexA8", toolChain:"GNU"},
];

var filterA9 = [
    {deviceFamily:"ARM", deviceVariant:"CortexA9", toolChain:"TI"},
];

var filterA9GNU = [
    {deviceFamily:"ARM", deviceVariant:"CortexA9", toolChain:"GNU"},
];

var filterA15 = [
    {deviceFamily:"ARM", deviceVariant:"CortexA15", toolChain:"TI"},
];

var filterA15GNU = [
    {deviceFamily:"ARM", deviceVariant:"CortexA15", toolChain:"GNU"},
];

/*
 * Filter on ~F28M3 since filtering for TMS320C/F prefixes isn't
 * working.  We need to get a precise deviceId list from Toronto so we can
 * determine how to make these filters more reliable and sustainable.
 */
var filterC28 = [
    {deviceFamily: "C2000", deviceId: "~.*(F28M3|Generic|G00883).*", toolChain: "TI"},
];

var defaultFilterMSP430 = [
    {deviceId: "MSP430F5.*", toolChain: "TI"},
    {deviceId: "MSP430F6.*", toolChain: "TI"},
    {deviceId: "MSP430BT.*", toolChain: "TI"},
    {deviceId: "CC430.*",    toolChain: "TI"},
];

var filterMSP430 = [
];

var filterStellarisM3 = [
    {deviceFamily:"ARM", deviceId:".*LM3.*", toolChain:"TI"},
];

var filterStellarisM3GNU = [
    {deviceFamily:"ARM", deviceId:".*LM3.*", toolChain:"GNU"},
];

var filterTivaM4 = [
    {deviceFamily:"ARM", deviceId:".*TM4C.*", toolChain:"TI"},
    {deviceFamily:"ARM", deviceId:".*TM4L.*", toolChain:"TI"},
    {deviceFamily:"ARM", deviceId:".*LM4.*", toolChain:"TI"},
    {deviceFamily:"ARM", deviceId:".*TM4E.*", toolChain:"TI"},
];

var filterTivaM4GNU = [
    {deviceFamily:"ARM", deviceId:".*TM4C.*", toolChain:"GNU"},
    {deviceFamily:"ARM", deviceId:".*TM4L.*", toolChain:"GNU"},
    {deviceFamily:"ARM", deviceId:".*LM4.*", toolChain:"GNU"},
    {deviceFamily:"ARM", deviceId:".*TM4E.*", toolChain:"GNU"},
];

var filterCC32xxM4 = [
    {deviceFamily:"ARM", deviceId:".*CC32.*", toolChain:"TI"},
];

var filterCC26xxM3 = [
    {deviceFamily:"ARM", deviceId:".*CC26.*", toolChain:"TI"},
];

var filterConcertoM3 = [
    {deviceFamily:"ARM", deviceId:".*F28M3.*", toolChain:"TI"},
];

var filterConcertoC28 = [
    {deviceFamily:"C2000", deviceId:".*F28M3.*", toolChain:"TI"},
];

var filterTI816X_DSP = [
    {deviceFamily: "C6000",
     deviceId: ".*(TI816x).*",
     toolChain:"TI"},
];

/*
 *  TODO: This should filter out everything but TI816X M3. As it is,
 *  the Cortex A is not filtered out.
 */
var filterTI816X_M3 = [
    {deviceFamily: "ARM", deviceId: ".*(TI816x).*", toolChain:"TI"},
];

var filterC6472 = [
    {deviceFamily:"C6000", deviceId:".*6472.*", toolChain:"TI"},
];

/*
 * linkerCommandFile is used below since CCS/project wizard does not allow the
 * use of linkerCommandFile like we'd like.  The workaround is to provide
 * the .cmd file in the fileList[] array and set linkerCommandFile to "".
 */
var options = [
    /* Generics */
    {
        variant: "genericARM",
        filter: filterGenericARM,
        cfgPrefix: "",
        linkerCommandFile: "",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: ""
    },
    {
        variant: "genericC6000",
        filter: filterGenericC6000,
        cfgPrefix: "",
        linkerCommandFile: "",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "C6000"
    },
    /* A8 */
    {
        variant: "a8",
        filter: filterA8,
        cfgPrefix: "",
        linkerCommandFile: "",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "Sitara"
    },
    /* A8 GNU */
    {
        variant: "a8GNU",
        filter: filterA8GNU,
        cfgPrefix: "",
        target: "gnu.targets.arm.A8F",
        linkerCommandFile: "",
        compilerBuildOptions: "-mfloat-abi=hard",
        linkerBuildOptions: "-nostartfiles -static --gc-sections -lgcc -lc -lm -lnosys -L${BIOS_CG_ROOT}/packages/gnu/targets/arm/libs/install-native/arm-none-eabi/lib/fpu",
        productGroup: "Sitara"
    },
    /* A9 GNU */
    {
        variant: "a9GNU",
        filter: filterA9GNU,
        cfgPrefix: "",
        target: "gnu.targets.arm.A9F",
        linkerCommandFile: "",
        compilerBuildOptions: "-mfloat-abi=hard",
        linkerBuildOptions: "-nostartfiles -static --gc-sections -lgcc -lc -lm -lnosys -L${BIOS_CG_ROOT}/packages/gnu/targets/arm/libs/install-native/arm-none-eabi/lib/fpu",
        productGroup: "Sitara"
    },
    /* A15 GNU */
    {
        variant: "a15GNU",
        filter: filterA15GNU,
        cfgPrefix: "",
        target: "gnu.targets.arm.A15F",
        linkerCommandFile: "",
        compilerBuildOptions: "-mfloat-abi=hard",
        linkerBuildOptions: "-nostartfiles -static --gc-sections -lgcc -lc -lm -lnosys -L${BIOS_CG_ROOT}/packages/gnu/targets/arm/libs/install-native/arm-none-eabi/lib/fpu",
        productGroup: "Sitara"
    },
    /* C28 */
    {
        variant: "c28",
        filter: filterC28,
        cfgPrefix: "small/",
        platform: "ti.platforms.tms320x28:$DeviceId$",
        linkerCommandFile: "ti/platforms/tms320x28/include/$DeviceId$.cmd",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "C2000"
    },
    /* Concerto C28 */
    {
        variant: "c28Concerto",
        filter: filterConcertoC28,
        cfgPrefix: "small/",
        platform: "ti.platforms.concertoC28:$DeviceId$",
        linkerCommandFile: "ti/platforms/concertoC28/include/$DeviceId$.cmd",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "C2000"
    },
    /* Concerto M3 */
    {
        variant: "m3Concerto",
        filter: filterConcertoM3,
        cfgPrefix: "small/",
        platform: "ti.platforms.concertoM3:$DeviceId$",
        linkerCommandFile: "ti/platforms/concertoM3/include/$DeviceId$.cmd",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "C2000"
    },
    /* CC2650 */
    {
        variant: "cc26xxM3",
        filter: filterCC26xxM3,
        cfgPrefix: "small/",
        platform: "ti.platforms.cc26xx:$DeviceId$",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "cc2xxx"
    },
    /* CC3200 */
    {
        variant: "cc32xxM4",
        filter: filterCC32xxM4,
        cfgPrefix: "small/",
        platform: "ti.platforms.tiva:$DeviceId$",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "cc3xxx"
    },
    /* MSP430 */
    {
        variant: "msp430",
        filter: filterMSP430,
        cfgPrefix: "msp430/",
        platform: "ti.platforms.msp430:$DeviceId$",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "MSP430"
    },
    /* Stellaris M3 */
    {
        variant: "stellarisM3",
        filter: filterStellarisM3,
        cfgPrefix: "small/",
        platform: "ti.platforms.stellaris:$DeviceId$",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: ""
    },
    /* Stellaris M3 GNU - Not ready yet.
    {
        variant: "m3GNU",
        filter: filterStellarisM3GNU,
        cfgPrefix: "small/",
        target: "gnu.targets.arm.M3",
        platform: "ti.platforms.tiva:$DeviceId$",
        compilerBuildOptions: "-mthumb -march=armv7-m",
        linkerBuildOptions: "-nostartfiles -static --gc-sections -lgcc -lc -lm -L${BIOS_CG_ROOT}/packages/gnu/targets/arm/libs/install-native/arm-none-eabi/lib/armv7-m"
    }, */
    /* TIVA */
    {
        variant: "tivaM4",
        filter: filterTivaM4,
        cfgPrefix: "small/",
        platform: "ti.platforms.tiva:$DeviceId$",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "TivaC"
    },
    /* Tiva GNU */
    {
        variant: "tivaM4GNU",
        filter: filterTivaM4GNU,
        cfgPrefix: "small/",
        target: "gnu.targets.arm.M4F",
        platform: "ti.platforms.tiva:$DeviceId$",
        compilerBuildOptions: "-mthumb -march=armv7e-m -mfloat-abi=hard -mfpu=fpv4-sp-d16",
        linkerBuildOptions: "-nostartfiles -static --gc-sections -lgcc -lc -lm -lnosys -L${BIOS_CG_ROOT}/packages/gnu/targets/arm/libs/install-native/arm-none-eabi/lib/armv7e-m/fpu",
        productGroup: "TivaC"
    },
    /* TI816X DSP */
    {
        variant: "ti816xDSP",
        filter: filterTI816X_DSP,
        cfgPrefix: "",
        target: "ti.targets.elf.C674",
        platform: "ti.uia.examples.platforms.evmTI816X.dsp",
        linkerCommandFile: "",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: ""
    },
    /* c6472 */
    {
        variant: "c6472",
        filter: filterC6472,
        cfgPrefix: "",
        target: "ti.targets.C64P",
        platform: "ti.platforms.evm6472",
        linkerCommandFile: "",
        compilerBuildOptions: "",
        linkerBuildOptions: "",
        productGroup: "C6000"
    },
];

/*
 *  ======== makeFilterMSP430 ========
 *  Generate the MSP430 filter based on the devices listed in msp430Settings.xs.
 */
function makeFilterMSP430()
{
    try {
        var msp430 =
            xdc.loadCapsule("ti/sysbios/family/msp430/msp430Settings.xs");

        var devices = msp430.devices[0];

        for (var i = 1; i < msp430.devices.length; i++) {
            devices += "|" + msp430.devices[i];
        }
        filterMSP430.push({deviceId:devices, toolChain:"TI"});
    }
    catch (e) {
        filterMSP430 = defaultFilterMSP430;
    }
}
