var deviceCapsule = xdc.loadCapsule("Devices.xs");
var exampleCapsule = xdc.loadCapsule("ExampleList.xs");

var productGroup = environment["productGroup"];
var tirtosSupport = environment["tirtosSupport"];

var devices                     = deviceCapsule.options;
var singleCoreExamples          = exampleCapsule.singleCoreExamples;
var NDKExamples                 = exampleCapsule.NDKExamples;
var MessageQExamples            = exampleCapsule.MessageQExamples;
var EVMTI816X_DSPExamples       = exampleCapsule.EVMTI816X_DSPExamples;
var EVMTI816X_VideoM3Examples   = exampleCapsule.EVMTI816X_VideoM3Examples;
var EVMTI816X_VpssM3Examples    = exampleCapsule.EVMTI816X_VpssM3Examples;
var examplesC6000               = exampleCapsule.examplesC6000;
var c28Examples                 = exampleCapsule.c28Examples;
var msp430Examples              = exampleCapsule.msp430Examples;
var rootString;

/*
 *  ======== module$meta$init ========
 */
function module$meta$init()
{
    var Examples = this;
    var curGroup = "";

    deviceCapsule.makeFilterMSP430();

    if (tirtosSupport == "false") {
        rootString = "ti.uia.examples.root";
        Examples.templateGroupArr.$add({
            id          : rootString,
            name        : "System Analyzer (UIA)",
            description : "System Analyzer (UIA) Tutorials, Templates and Examples"
        });
    }
    else {
        rootString = "examples.root_" + productGroup;
        Examples.templateGroupArr.$add({
            id          : rootString,
            name        : "TI-RTOS for " + productGroup,
            description : "TI-RTOS Templates and Examples"
        });
    }

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia",
        name        : "Instrumentation Examples",
        description : "Instrumentation Example Projects",
        groups      : [rootString]
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.singlecore",
        name        : "Single-core Examples",
        description : "System Analyzer (UIA) Single-core Examples",
        groups      : [rootString + ".uia"]
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.multicore",
        name        : "Multi-core Examples",
        description : "System Analyzer (UIA) Multi-core Examples",
        groups      : [rootString + ".uia"]
    });

    /* Enable this code when GNU target builds are working */
if (1) {
    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.targetsTI",
        name        : "TI Target Examples",
        description : "System Analyzer (UIA) Examples for TI targets.",
        groups      : [rootString + ".uia"],
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.targetsGNU",
        name        : "GNU Target Examples",
        description : "System Analyzer (UIA) Examples for GNU targets.",
        groups      : [rootString + ".uia"],
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.targetsTI.singlecoreTI",
        name        : "Single-core Examples",
        description : "System Analyzer (UIA) Single-core Examples",
        groups      : [rootString + ".uia.targetsTI"],
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.targetsGNU.singlecoreGNU",
        name        : "Single-core Examples",
        description : "System Analyzer (UIA) Single-core Examples",
        groups      : [rootString + ".uia.targetsGNU"],
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.targetsTI.multicoreTI",
        name        : "Multi-core Examples",
        description : "System Analyzer (UIA) Multi-core Examples",
        groups      : [rootString + ".uia.targetsTI"]
    });

    Examples.templateGroupArr.$add({
        id          : rootString + ".uia.targetsGNU.multicoreGNU",
        name        : "Multi-core Examples",
        description : "System Analyzer (UIA) Multi-core Examples",
        groups      : [rootString + ".uia.targetsGNU"]
    });
}
    for (var i = 0; i < devices.length; i++) {
        if ((productGroup != "ALL") &&
                (productGroup != devices[i].productGroup)) {
                continue;
        }

        // Single core examples - Add these for all devices, except for the
        // ones whose filters overlap with "generic". This prevents them
        // from being added multiple times.
        //if (!devices[i].variant.match(/ti816xDSP|videoM3|vpssM3|genericC6000|c6472/)) {
        //    addExamples(Examples, devices[i], singleCoreExamples);
        //}

        if (devices[i].variant.match(/GNU/)) {
            /* GNU ARM targets */
            curGroup = "GNU";
        }
        else if ((devices[i].variant == "a8") ||
                 (devices[i].variant == "a9") ||
                 (devices[i].variant == "a15") ||
                 (devices[i].variant == "c28") ||
                 (devices[i].variant == "c28Concerto") ||
                 (devices[i].variant == "cc26xxM3") ||
                 (devices[i].variant == "cc32xxM4") ||
                 (devices[i].variant == "m3Concerto") ||
                 (devices[i].variant == "msp430") ||
                 (devices[i].variant == "stellarisM3") ||
                 (devices[i].variant == "tivaM4") ||
                 (devices[i].variant == "tivaCM4")) {
            /* TI ARM targets */
            curGroup = "TI";
        }
        else {
            curGroup = "";
        }

        /*
         *  Add single core examples for all devices. The filters for the
         *  devices must not overlap, or these examples will be added
         *  twice. The videoM3 and vpssM3 devices for TI816X use the same
         *  filter, though, so we have to handle that case.
         */
        if (!devices[i].variant.match(/vpssM3/)) {
            addExamples(Examples, devices[i], singleCoreExamples, curGroup);
        }

        if (devices[i].variant.match(/genericC6000|c6472|ti816xDSP/)) {
            addExamples(Examples, devices[i], examplesC6000, curGroup);
        }

        /* Add NDK examples for generic and m3 (Stellaris, Concerto) */
        if (devices[i].variant == "c6472") {
            addExamples(Examples, devices[i], NDKExamples, curGroup);
            addExamples(Examples, devices[i], MessageQExamples, curGroup);
        }

        /* Add c28 specific examples */
        if (devices[i].variant == "c28") {
            addExamples(Examples, devices[i], c28Examples, curGroup);
        }

        /* Add MSP430 specific examples */
        if (devices[i].variant == "msp430") {
            addExamples(Examples, devices[i], msp430Examples, curGroup);
        }

        // Add the TI816X examples
        if (devices[i].variant == "ti816xDSP") {
            addExamples(Examples, devices[i], EVMTI816X_DSPExamples, curGroup);
        }

        if (devices[i].variant == "videoM3") {
            addExamples(Examples, devices[i], EVMTI816X_VideoM3Examples, curGroup);
        }

        if (devices[i].variant == "vpssM3") {
            addExamples(Examples, devices[i], EVMTI816X_VpssM3Examples, curGroup);
        }
    }

    /* add common properties shared by all examples */
    for (var i = 0; i < Examples.templateArr.length; i++) {
        var ex = Examples.templateArr[i];

        /* add debugability to all examples using TI toolchain */
        var debug = true;
        for (var j = 0; j < ex.filterArr.length; j++) {
            if (ex.filterArr[j].toolChain != "TI") {
                debug = false;
            }
        }
        if (debug) {
            ex.configuroOptions =
                "--compileOptions &quot;-g --optimize_with_debug&quot;";
        }
    }
}

/*
 *  ======== addExamples ========
 */
function addExamples(Examples, opts, examples, curGroup)
{
    for (var j = 0; j < examples.length; j++) {

        var group = examples[j].groups;
        if (curGroup.match(/TI/)) {
            group = examples[j].groupsTI;
        }
        else if (curGroup.match(/GNU/)) {
            group = examples[j].groupsGNU;
        }

        var requiredProducts;
        if (tirtosSupport == "false"){
            requiredProducts = examples[j].requiredProducts;
        }
        Examples.templateArr.$add({
                buildProfile: "release",
                filterArr: opts.filter,
                target: opts.target,
                platform: opts.platform,
                linkerCommandFile: opts.linkerCommandFile,
                compilerBuildOptions: opts.compilerBuildOptions,
                linkerBuildOptions: opts.linkerBuildOptions,
                title: examples[j].title,
                name: examples[j].name,
                description: examples[j].description,
                fileList: [
                    {path: examples[j].root + examples[j].cFile},
                    {path: examples[j].root + opts.cfgPrefix +
                               examples[j].cfgFile}
                ],
                isHybrid: examples[j].isHybrid,
                groups: group,
                requiredProducts: requiredProducts,
                options: examples[j].options,
            });

        var ex = Examples.templateArr[Examples.templateArr.length - 1];

        /* Add any aditional files to the list */
        for (k = 0; k < examples[j].fileList.length; k++) {
            ex.fileList.$add({path: examples[j].root + examples[j].fileList[k]});
        }
    }
}

/*
 *  ======== makeFilterMSP430 ========
 *  Generate the MSP430 filter based on the devices listed in msp430Settings.xs.
 */
function makeFilterMSP430()
{
    try {
        var msp430 =
            xdc.loadCapsule("ti/sysbios/family/msp430/msp430Settings.xs");

        var devices = msp430.devices[0];

        for (var i = 1; i < msp430.devices.length; i++) {
            devices += "|" + msp430.devices[i];
        }
        filterMSP430.push({deviceId:devices, toolChain:"TI"});
    }
    catch (e) {
        filterMSP430 = defaultFilterMSP430;
    }
}
